


#include <stdio.h>
#include <stdlib.h>

#include <sys/timeb.h>
#include <string.h>
#include <ctype.h>
#if !defined(__GNUC__)
#include <conio.h>
#include <windows.h>
#else
#include <unistd.h>
#define __forceinline inline 
#endif


/*****  DEFINICIONES, SIMBOLOS,VARIABLES y CONSTANTES. *******/

//** La profundidad, (en nuestro caso) siempre alcanza mayor valor
//** que Ply, que es la iteracion.
#define PROF_MAXIMA      64


//** Maximo numero de movimientos que puede haber en la Lista
//** de movimientos generados, si se supera dara error.
#define MAX_MOV  3000

/*** Definicion de los diferentes tipos de fichas, con este sistema podemos
 *** saber si una ficha es blanca o negra:
 *** 		Ficha = Contenido[casilla]
 ***		if(Ficha > 0) color = blanco; else color = negro    */

#define R_N   -6
#define D_N   -5
#define T_N   -4
#define A_N   -3
#define C_N   -2
#define P_N   -1
#define VACIO  0
//#define ZERO   0
#define P_B    1
#define C_B    2
#define A_B    3
#define T_B    4
#define D_B    5
#define R_B    6
#define FUERA 100

#if defined(__GNUC__)
#include <stdint.h>
typedef uint64_t U64;
typedef uint32_t U32;
typedef uint8_t   U8;
#else
typedef unsigned __int64 U64;
typedef unsigned __int32 U32;
typedef unsigned __int8   U8;
#endif

// TTables
#define Hash_EXACT   1
#define Hash_ALPHA   2
#define Hash_BETA    3
#define Hash_DESCONOCIDO 4

//*** Valor de las piezas, es suceptible de ser cambiados
#define VAL_P    100
#define VAL_C    320
#define VAL_A    330
#define VAL_T    520
#define VAL_D    990
#define VAL_R   2000

//*** El valor del mate es el mximo valor posible
#define VALMATE  32000
#define VAL_WIN  10000
#define VAL_NUL  32016
// Para los extensores
#define UNPLY         2
#define MEDIO_PLY     1

//*** Al almacenar las jugadas les daremos un valor para poder clasificarlas,
//*** estos son las bonificaciones que daremos a las jugadas mas importantes
#define BONUS_HASH  1900000
#define BONUS_VP    1880000
#define BONUS_MKIL  1860000
#define BONUS_PDAMA 1850000
#define BONUS_BCAP  1850000
#define BONUS_CAP   1840000
#define BONUS_ALPASO 1840000
#define BONUS_KIL1  1830000
#define BONUS_PCAB  1820000
#define BONUS_KIL2  1810000
#define BONUS_MCAP  1809000 
#define MAX_HIST    1800000

/*** Determinamos en que momento de la partida estamos, por el material
 *** que queda en el tablero, tendremos que hacer mas apartados, tales
 *** como APERTURA, MJUEGO1, MJUEGO2, FINAL1, FINAL2, FINALPEONES ETC.. */
#define APERTURA  ((4*(VAL_T+VAL_A+VAL_C))+(2*VAL_D)+(16*VAL_P))
//#define FASEFINAL (4*VAL_T)+(2*VAL_A)

//*** Definimos las casillas para poder trabajar mejor con ellas.
//*** if(contenido[E1]==R_B) ..se entiende mejor que if(contenido[25]==6),
//*** aunque las dos sentencias son correctas y significan lo mismo.
//*** Ademas con este sistema es mas legible cuando recorremos el tablero:
//*** for(casilla = A1; casilla <= H8; casilla++);


enum{A1=21,B1,C1,D1,E1,F1,G1,H1,
	 A2=31,B2,C2,D2,E2,F2,G2,H2,
	 A3=41,B3,C3,D3,E3,F3,G3,H3,
	 A4=51,B4,C4,D4,E4,F4,G4,H4,
	 A5=61,B5,C5,D5,E5,F5,G5,H5,
	 A6=71,B6,C6,D6,E6,F6,G6,H6,
	 A7=81,B7,C7,D7,E7,F7,G7,H7,
	 A8=91,B8,C8,D8,E8,F8,G8,H8,H9};


#ifndef true
	#define true  1
	#define false 0
	#define bool  U8
#endif

#define NO_ENROQUE   0
#define ENROQUECORTO 1
#define ENROQUELARGO 2

#define BLANCAS      1
#define NEGRAS      -1
#define B            0
#define N            1
#define desconocido  2
#define ILEGAL       0

#define Min(x, y) (((x) < (y))? (x) : (y))
#define Max(x, y) (((x) < (y))? (y) : (x))
#define es_casilla(x) contenido[x] != FUERA
//#define es_deslizante(x) abs(x) > 2 && abs(x) < 6

/***********  TIPO DE DATOS ESTRUCTURADOS.   ***********/

typedef struct {
   int origen;         // Casilla origen
   int destino;        // Casilla destino
   int capturada;      // Contenido de la casilla destino, antes de mover,
   int promocion;      // Para saber a que se promueve
   int enroquetipo;    // Enroque Corto, Enroque largo o No_Enroque
   int cap_alpaso;     // Por si se trata de una captura al paso
   int eval;           // Evaluacion para esa jugada en concreto
} tipojugada;


typedef struct {
   int origen;
   int destino;
   bool capturada;
   U8   ficha;
} tipomov;

//*** Para ordenar bien los movimientos generados, necesitamos
//*** saber de que movimiento se trata

tipomov matekiller[64];
tipomov killer1[64];
tipomov killer2[64];
int     good_mov[3][100][100];
int     max_good_mov;
tipomov hashmov = {0,0};
tipomov nomove  = {0,0};

// para el control de los movimientos hechos
typedef struct {
   U64 hash;
   int fifty;
   int origen;
   int destino;
   int cap_alpaso;
   int alpaso;
   int capturada;
   int promovida;
   int enrocado;
}history;

//* Estructura de las hash para search(), los datos que guardamos son:
//* un numero asociado a la posicion
//* tipo del valor guardado
//* la profundidad en la que nos encontrabamos al guardar los datos
//* el bando que tenia el turno en ese momento y
//* el mejor movimiento

typedef struct {
	U64 Hash;
	int Type;
	int Depth;
	int Valor;
	int age;
	tipomov Mov;
} TType;

//* Estructura de las hash para eval(), los datos que guardamos son:
//* un numero asociado a la posicion
//* el valor real de la posicion y
//* el bando que tenia el turno en ese momento y
typedef struct {
	U64 Hash;
	int Valor;
	int Color;
} EType;

typedef struct
{ int    Type;
  tipomov Bestmove;
  int       OnMove;
  U64 Hash;
  int Bound;
}
QType;

typedef char *str;

/**************** DEFINICION DE VARIABLES GLOBALES **************/

char       NOMBRE[]="Betsabe_II";
char       VERSION[]="2023";
int        contenido[120];
int        Alpaso;
bool       Analiza;
bool       ponderar;
bool       en_tiempo_contrario;
int		   Trans_Age;
int        Movida[H8+1];
bool       enrocado_B;
bool       enrocado_N;
int        Index;
int        Ply;
int		   Eval_Last_Move;
bool       Var_Principal;
int        maxtiempo,quedan;
int        start_time, stop_time, stop_search;
int        PilaMax[PROF_MAXIMA];//almacen de jugadas generadas

// Contadores para estadistica
unsigned long int   Nodos;
unsigned long int   NulProb,NulExito;
unsigned long int   Futil,LMR,Prueba;
unsigned long int   Hash_Probe;
unsigned long int   Hash_Hits;
unsigned long int   PosC,PosL,PosH;
unsigned long int   Razor,Estatic;
unsigned long int   Extensores,Ext_jaque;

// Opciones de busqueda y evaluacion, esto en un futuro,
// tendria que estar en un fichero ini.

bool       USAR_LAZY    = true;
bool       USAR_HASHEV  = true;

tipomov     VarPrinc[PROF_MAXIMA][PROF_MAXIMA];
tipojugada  ListaMov[MAX_MOV];


/*** Tablero de 10x12 con su contenido, seria suficiente con un tablero
 *** de 10 X 10 (8 X 8 mas los bordes para saber cuando acaba el tablero),
 *** pero como el caballo puede saltar 2 casillas, se podra dar el caso de
 *** saltar a una casilla inexistente; tampoco es necesario un tablero de
 *** 12 X 12, porque los bordes laterales son contiguos; lo explicaremos con
 *** mas detalle al comentar el generador de movimientos.

 *** Este es el aspecto interno que tiene nuestro tablero real, tal como lo ve
 *** el programa, el cuadrado central representa el tablero de juego, y los
 *** circundantes son las casillas del borde:

	000   001 002 003 004 005 006 007 008   009   OUT..FUERA
	010   011 012 013 014 015 016 017 018   019   OUT..FUERA

	020   021 022 023 024 025 026 027 028   029   A1...H1
	030   031 032 033 034 035 036 037 038   039   A2...H2
	040   041 042 043 044 045 046 047 048   049   A3...H3
	050   051 052 053 054 055 056 057 058   059   A4...H4
	060   061 062 063 064 065 066 067 068   069   A5...H5
	070   071 072 073 074 075 076 077 078   079   A6...H6
	080   081 082 083 084 085 086 087 088   089   A7...H7
	090   091 092 093 094 095 096 097 098   099   A8...H8

	100   101 102 103 104 105 106 107 108   109   FUERA..FUERA
	110   111 112 113 114 115 116 117 118   119   FUERA..FUERA

 *** Este es el mismo tablero anterior, pero utilizando simbolos, y con su
 *** contenido inicial:  */

const int Tablero_Inicio[120] =  {
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA,
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA,
FUERA,  T_B,    C_B ,   A_B ,   D_B ,   R_B ,   A_B ,   C_B ,   T_B,  FUERA,
FUERA,  P_B,    P_B ,   P_B ,   P_B ,   P_B ,   P_B ,   P_B ,   P_B,  FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA,  P_N ,   P_N ,   P_N ,   P_N ,   P_N ,   P_N ,   P_N ,   P_N,  FUERA,
FUERA,  T_N,    C_N ,   A_N ,   D_N ,   R_N ,   A_N ,   C_N ,   T_N,  FUERA,
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA,
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA};

//* Lo utilizamos cuando introducimos una posicion poder retirar todas
//* fichas del tablero
const int Tablero_Vacio[120] =  {
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA,
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO,  VACIO, FUERA,
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA,
FUERA, FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA,  FUERA, FUERA};

const int BOARD_64[64] = {
	A1,B1,C1,D1,E1,F1,G1,H1,
	A2,B2,C2,D2,E2,F2,G2,H2,
	A3,B3,C3,D3,E3,F3,G3,H3,
	A4,B4,C4,D4,E4,F4,G4,H4,
	A5,B5,C5,D5,E5,F5,G5,H5,
	A6,B6,C6,D6,E6,F6,G6,H6,
	A7,B7,C7,D7,E7,F7,G7,H7,
	A8,B8,C8,D8,E8,F8,G8,H8};
/*
const int BOARD_36[36] = {
	B2,C2,D2,E2,F2,G2,
	B3,C3,D3,E3,F3,G3,
	B4,C4,D4,E4,F4,G4,
	B5,C5,D5,E5,F5,G5,
	B6,C6,D6,E6,F6,G6,
	B7,C7,D7,E7,F7,G7};
*/
const int BOARD_16[16] = {
	C3,D3,E3,F3,
	C4,D4,E4,F4,
	C5,D5,E5,F5,
	C6,D6,E6,F6};

//* Para saber por que tipo de casillas circulan los alfiles
 const int COLOR_CA[100] ={
//   A  B  C  D  E  F  G  H
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 1, 0, 1, 0, 1, 0, 1, 0, 0,  // A1 ... H1
  0, 0, 1, 0, 1, 0, 1, 0, 1, 0,
  0, 1, 0, 1, 0, 1, 0, 1, 0, 0,
  0, 0, 1, 0, 1, 0, 1, 0, 1, 0,
  0, 1, 0, 1, 0, 1, 0, 1, 0, 0,
  0, 0, 1, 0, 1, 0, 1, 0, 1, 0,
  0, 1, 0, 1, 0, 1, 0, 1, 0, 0,
  0, 0, 1, 0, 1, 0, 1, 0, 1, 0};   // A8 ... H8

//* Para las negras sera, fila = (9-fila)
 const int FILA[100] ={
//   A  B  C  D  E  F  G  H
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 1, 1, 1, 1, 1, 1, 1, 1, 0,  // A1 ... H1
  0, 2, 2, 2, 2, 2, 2, 2, 2, 0,
  0, 3, 3, 3, 3, 3, 3, 3, 3, 0,
  0, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  0, 5, 5, 5, 5, 5, 5, 5, 5, 0,
  0, 6, 6, 6, 6, 6, 6, 6, 6, 0,
  0, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  0, 8, 8, 8, 8, 8, 8, 8, 8, 0} ;  // A8 ... H8


 const int COLUMNA[100] ={
// Ponemos 2 columnas mas, una a cada lado
// para facilitar las comparaciones.
//   A  B  C  D  E  F  G  H
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,  //A1 ... H1
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9} ;   // A8 ... H8

 const int DIAGONAL1[100] ={
//   A  B  C  D  E  F  G  H
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 0,  // A1 ... H1
  0, 2, 3, 4, 5, 6, 7, 8, 9, 0,
  0, 3, 4, 5, 6, 7, 8, 9,10, 0,
  0, 4, 5, 6, 7, 8, 9,10,11, 0,
  0, 5, 6, 7, 8, 9,10,11,12, 0,
  0, 6, 7, 8, 9,10,11,12,13, 0,
  0, 7, 8, 9,10,11,12,13,14, 0,
  0, 8, 9,10,11,12,13,14,15, 0} ;  // A8 ... H8

   const int DIAGONAL2[100] ={
//   A  B  C  D  E  F  G  H
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 8, 9,10,11,12,13,14,15, 0,  // A1 ... H1
  0, 7, 8, 9,10,11,12,13,14, 0,
  0, 6, 7, 8, 9,10,11,12,13, 0,
  0, 5, 6, 7, 8, 9,10,11,12, 0,
  0, 4, 5, 6, 7, 8, 9,10,11, 0,
  0, 3, 4, 5, 6, 7, 8, 9,10, 0,
  0, 2, 3, 4, 5, 6, 7, 8, 9, 0,
  0, 1, 2, 3, 4, 5, 6, 7, 8, 0} ;  // A8 ... H8

// Para saber en que casilla corona un peon blanco
 const int CORONA_B[100] ={
//   A  B  C  D  E  F  G  H
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1
  0, A8, B8, C8, D8, E8, F8, G8, H8, 0,
  0, A8, B8, C8, D8, E8, F8, G8, H8, 0,
  0, A8, B8, C8, D8, E8, F8, G8, H8, 0,
  0, A8, B8, C8, D8, E8, F8, G8, H8, 0,
  0, A8, B8, C8, D8, E8, F8, G8, H8, 0,
  0, A8, B8, C8, D8, E8, F8, G8, H8, 0,
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;  // A8 ... H8


// Para saber en que casilla corona un peon negro
 const int CORONA_N[100] ={
//   A  B  C  D  E  F  G  H
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1
  0, A1, B1, C1, D1, E1, F1, G1, H1, 0,
  0, A1, B1, C1, D1, E1, F1, G1, H1, 0,
  0, A1, B1, C1, D1, E1, F1, G1, H1, 0,
  0, A1, B1, C1, D1, E1, F1, G1, H1, 0,
  0, A1, B1, C1, D1, E1, F1, G1, H1, 0,
  0, A1, B1, C1, D1, E1, F1, G1, H1, 0,
  0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;   // A8 ... H8


//*** Tablas de incrementos para el generador de movimientos

const int Inc_Ca[8]  = {-21, -19, -12, -8, 8, 12, 19, 21};
const int Inc_Rey[8] = {1, 9, 10, 11, -1, -9, -10, -11};
const int Inc_Da[8]  = {1, -1, 9, -9, -10, 10, -11, 11};
const int Inc_To[4]  = {1, -1,  -10, 10};
const int Inc_Al[4]  = {9, -9, -11, 11};
//const int Inc_Pe[8]  = {-10, -20, -9, -11, 10, 20, 9, 11};

//** Para llevar control del valor material, fichas y peones
//** en todo momento
int     MaterialBB,FichasBB,PeonesBB;
int     MaterialNN,FichasNN,PeonesNN;

const int  ValFicha[7]= {0,VAL_P,VAL_C,VAL_A,VAL_T,VAL_D,VAL_R};
const int  ValFicha2[7]= {0,100,400,400,600,1200,5000}; // aqui Alfil y Caballo mismo varlor
//Para dibular el tablero en modo consola
const char SimboloFicha[7]    = { ' ','P','C','A','T','D','R' };
const char SimboloFichawb[7]  = { ' ','p','n','b','r','q','k' };
const char SimboloFichawb2[7] = { ' ',' ','N','B','R','Q','K' };

//** DECLARACION DE RUTINAS

//main
void  newgame(void);
void  Test();

//eval
int   evalua_posicion(int alpha, int beta);
int   Evalua_atrapada_B(void);
int   Evalua_atrapada_N(void);

//peones
void  init_peones(void);
int   EvaluaPeonB(int ca);
int   EvaluaPeonN(int ca);
int   EvaluaPeonBF(int ca);
int   EvaluaPeonNF(int ca);
void  init_pcsq(void);

//finales
int   ReySolitario(void);
int   EvaluaRPR (int turno);
int   Eval_mate_alfil_caballo();
int   AlfilPeon_Alfil(int turno);
int   Alfil_peon(int turno);
int   Dama_peon(int turno);
int   Eval_ficha_ficha(int turno);
int   RTP_RT(void);

//search
void  computer_mov(void);
int   search(int alpha, int beta, int Depth,bool hacer_nulo,int check);
int   qsearch(int alpha,int beta,int Depth);
int   root_search(int alpha,int beta,int Depth);
bool  tenemos_que_parar();
__forceinline bool  avance_peligroso(int destino);
__forceinline int see_after(int i);

//gen
void  genera_todo(void);
void  genera_capturas(void);
static int   tomanextmov(void);
static void  anota_jugada(int origen, int destino);
static void  anota_captura(int origen, int destino);
static void  anota_capturaq(int origen, int destino);
static void  anota_cap_alpaso(int origen, int destino, int ca_cap_alpaso);
static void  anota_promocion(int origen, int destino);
static void  anota_promocionq(int origen, int destino);
int   AlfilB_ataca(int casilla);
int   AlfilN_ataca(int casilla);
static int   casilla_atacada(int casilla, int turno);
static int   es_legal(void);
void  MakeMove(int MovAct);
void  retrocede(void);
void  ordena_lista_mov();

//see
int   see(int ca_or, int ca_dest);
__forceinline  bool  peon_defiende(int origen,int destino);
__forceinline  bool Mala_captura(int origen,int destino);

//util
void  ActVarPrincipal(int MovAct);
void  MuestraVarPrinc(int ply,int eval);
void  lee_comando(void);
int	  Distancia[100][100];
void  dibuja_tablero(void);
int   input_mov(str jugada);
int   ca_numerica(str caalg);
void  muestra_mov(int MovAct);
void  inputnivel(void);
int   CheckInput(void);
static int   GetTime();
void  setboard_fen(const char *cadena);
void  help(void);
str   input(str text, str comando);
str   ca_algebraica(int canum,str caalg);
int   Dist(int cas1, int cas2);

//xboard
void  xboard();

// hash
void init_hash();
unsigned int  Rand32 (void);
U64  Rand64 (void);
void set_hash();
bool  repeticion();
void clear_hash(void);
int  probe_hash(int depth, int *value, int beta, tipomov *hashmov, U8 *null_ok);
void store_hash(int depth, int value, int flag, tipomov mov);
void Get_hash_move(tipomov *hashmov);
void store_eval(int score, int turno);
int  probe_eval(int turno);
void alloc_hash(void);
void free_hash(void);

int      Pos_RB,Pos_RN;
int      Color;
int      Betsabe;
int      DosJugadores;
int      wboard = 0;
bool     modotest;
bool     fase_final;

int      MAX_PLY;
int      Profundidad;
int      DamasB,TorresB,AlfilesB,CaballosB,PeonesB;
int      DamasN,TorresN,AlfilesN,CaballosN,PeonesN;
int      DB, DN; // ubicacion de las Damas, utilizadas en eval()
// Tablas hash
int  TTSize;		// tamao para las hash search
int  ETSize;		// tamao para las hash eval
int  fifty,hply;

//Memoria de los movimientos jugados 
history    historial[2048];
U64        hash;
U64        hash_piece[14][100];
U64        zobrist[14][100];
U64        hash_turno;
U64        hash_alpaso[100];







