// **  peones.h

// Ultima revisin invierno 2017-2018
/*
 *******************************************************************************
 *                                                                             *
 *   LA EVALUACION DE LOS PEONES                                               *
 *                                                                             *
 *   Init_peones() se encarga de marcar en cada columna, la fila que ocupa el  *
 *   peon mas retrasado de cada bando; al inicializar los pone  a todo los     *
 *   blancos en la fila 8 y los negro en la 1, lugares imposible para los      *
 *   peones, por lo que si permanecen en esa fila, querra decir que no hay     *
 *   peon en esa columna.                                                      *
 *   Aprovechamos que recorremos todo el tablero, para hacer dos lista, una    *
 *   de las blancas y otra de las negras, con el num. de fichas y peones  y el *
 *   lugar que ocupan, luego sera algo mas rapido recorrer el tablero en busca *
 *   de fichas y peones.                                                       *
 *                                                                             *
 *   Tenemos dos rutinas para la evaluacion de los peones, una para la         *
 *   apertura y medio juego y otra para los finales, practimante son iguales,  *
 *   excepto en el tratamiento que le damos al peon pasado y al candidato que  *
 *   en los finales es mas elaborado.                                          *
 *                                                                             *
 *   1) Miramos si esta expuesto (no tiene un peon contrario que le haga       *
 *   frente, con lo cual esta expuesto a los ataques frontales, solo por esto  *
 *   no penalizamos.                                                           *
 *                                                                             *
 *   2) Miramos si es un peon doblado, solo podemos saber si tenemos otro peon *
 *   detras, pero no si tenemos alguno delante, por lo que solo vamos a        *
 *   penalizar al peon doblado mas retrasado.                                  *
 *                                                                             *
 *   3) Comprobamos si se encuentra aislado, los compaeros de los lados o han *
 *   sido capturados o se han doblado; penalizamos                             *
 *                                                                             *
 *   4) Peon duo, si tenemos un compaero justo al lado, eso te da mucho animo *
 *   para seguir avanzando; bonificamos.                                       *
 *                                                                             *
 *   5) Miramos si se trata de un peon retrasado; (se considera peon retrasado *
 *   al peon que sin estar aislado, no puede ser defendido por un compaero).  *
 *   El tema es bastante serio, justo delante del peon tenemos una casilla     *
 *   debil, que seguramente ocupara el contrario con una de sus fichas, y si   *
 *   ademas esta expuesto, se agrava el problema. De todas forma, no lo vamos  *
 *   a considerar retrasado, si no esta expuesto y ademas defiende a un        *
 *   compaero.                                                                *
 *                                                                             *
 *   6) Es hora de ver si tenmos un peon pasado.                               *
 *      a) Apertura y medio juego: solo nos interesan los pasado que ya se     *
 *         encuentren en la fila 4, si es asi le damos un bonus dependiendo de *
 *         la fila; si esta bloqueado,(no importa por quien), penalizamos.     *
 *         Tambien lo bonificamos si tiene un compaero al lado, o esta defen- *
 *         dido por otro peon, o no es aislado.                                *
 *      b) Finales:                                                            *
 *         -Si el contrario solo tiene una ficha y no es la Dama, el peon ya   *
 *          se encuentra adelantado, miramos la proximidad de los Reyes.       *
 *         -Si el contrario no tiene ficha, comprobamos si el peon puede coro- *
 *          nar solo, y marcamos la distancia hasta coronar, no importa que    *
 *          tengamos 4 que coronan solo, solo bonificaremos uno, el que corone *
 *          antes, y siempre que corone antes que el contrario.                *
 *         -Si no puede coronar solo, miramos si su Rey le asegura el camino.  *
 *         -Si tampoco tiene el pasillo, bonificamos si es un peon pasado      *
 *          alejado.                                                           *
 *         -Si el contrario tiene fichas, penalizamos que este bloqueado       *
 *          algo mas si por caballo o alfil, tambien bonificamos o penalizamos *
 *          dependiendo de que torre se encuentre detras del peon.             *
 *                                                                             *
 *   7) Si no es un peon pasado, miramos si tenemos un candidato, para ser     *
 *   candidato debe reunir 3 condiciones, estar expuesto, no ser aislado y no  *
 *   ser retrasado, lo bonificamos con medio bonus.                            *
 *******************************************************************************
 */

#define col_a 1
#define col_b 2
#define col_c 3
#define col_d 4
#define col_e 5
#define col_f 6
#define col_g 7
#define col_h 8

#define fila_1 1
#define fila_2 2
#define fila_3 3
#define fila_4 4
#define fila_5 5
#define fila_6 6
#define fila_7 7
#define fila_8 8

#define no_wpawn_in_col 8
#define no_bpawn_in_col 1

#define pb_controla(ca) ((contenido[ca - 9] == P_B) + (contenido[ca - 11] == P_B))
#define pn_controla(ca) ((contenido[ca + 9] == P_N) + (contenido[ca + 11] == P_N))

int wp_pcsq_mg[120];
int bp_pcsq_mg[120];
int wn_pcsq_mg[120];
int bn_pcsq_mg[120];
int wb_pcsq_mg[120];
int bb_pcsq_mg[120];
int wr_pcsq_mg[120];
int br_pcsq_mg[120];
int wq_pcsq_mg[120];
int bq_pcsq_mg[120];
int wk_pcsq_mg[120];
int bk_pcsq_mg[120];

//int wp_pcsq_eg[120];
//int bp_pcsq_eg[120];
int wn_pcsq_eg[120];
int bn_pcsq_eg[120];
int wb_pcsq_eg[120];
int bb_pcsq_eg[120];
int wr_pcsq_eg[120];
int br_pcsq_eg[120];
int wq_pcsq_eg[120];
int bq_pcsq_eg[120];
int wk_pcsq_eg[120];
int bk_pcsq_eg[120];

int FLIP[120]= {
   //    A   B    C    D    E    F    G    H  
   110, 111,112, 113, 114, 115, 116, 117, 118,119,
   100, 101,102, 103, 104, 105, 106, 107, 108,109,
   90,  91,  92,  93,  94,  95,  96,  97,  98, 99,  // A8 ... H8  
   80,  81,  82,  83,  84,  85,  86,  87,  88, 89, 
   70,  71,  72,  73,  74,  75,  76,  77,  78, 79,  
   60,  61,  62,  63,  64,  65,  66,  67,  68, 69,  
   50,  51,  52,  53,  54,  55,  56,  57,  58, 59,  
   40,  41,  42,  43,  44,  45,  46,  47,  48, 49, 
   30,  31,  32,  33,  34,  35,  36,  37,  38, 39,  
   20,  21,  22,  23,  24,  25,  26,  27,  28, 29,  // A1 ... H1 
   10,  11,  12,  13,  14,  15,  16,  17,  18, 19,
   0,    1,   2,   3,   4,   5,   6,   7,   8,  9} ;

const int outpost[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A2 ... H2 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A3 ... H3 
   0,  0,  0,  8, 10, 10,  8,  0,  0, 0,  // A4 ... H4 
   0,  0,  4,  8, 16, 16,  8,  4,  0, 0,  // A5 ... H5 
   0,  0,  4, 10, 16, 16, 10,  4,  0, 0,  // A6 ... H6 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A7 ... H7 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int outpost_alf[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A2 ... H2 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A3 ... H3 
   0,  0,  0,  6, 10, 10,  6,  0,  0, 0,  // A4 ... H4 
   0,  0,  3,  6, 10, 10,  6,  3,  0, 0,  // A5 ... H5 
   0,  0,  3,  8, 10, 10,  8,  3,  0, 0,  // A6 ... H6 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A7 ... H7 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int control_centro[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A2 ... H2 
   0,  0,  0,  2,  2,  2,  2,  0,  0, 0,  // A3 ... H3 
   0,  0,  0,  2,  4,  4,  2,  0,  0, 0,  // A4 ... H4 
   0,  0,  0,  2,  4,  4,  2,  0,  0, 0,  // A5 ... H5 
   0,  0,  0,  2,  2,  2,  2,  0,  0, 0,  // A6 ... H6 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A7 ... H7 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

//BETSABE  
const int pawn_pcsq_mg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1  
   0,  5,  5,  5,-20,-20,  5,  5,  5, 0,  // A2 ... H2 
   0, -2, -2, -2, 10, 10, -2, -2, -2, 0,  // A3 ... H3 
   0,  0,  0,  0, 25, 25,  0,  0,  0, 0,  // A4 ... H4 
   0,  2,  2, 12, 16, 16, 12,  2,  2, 0,  // A5 ... H5 
   0,  4,  4, 12, 16, 16, 12,  4,  4, 0,  // A6 ... H6 
   0,  4,  4, 12, 16, 16, 12,  4,  4, 0,  // A7 ... H7 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;
   
/*
   const int pawn_pcsq_eg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A1 ... H1  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A2 ... H2 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A3 ... H3 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A4 ... H4 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A5 ... H5 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A6 ... H6 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A7 ... H7 
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

*/

const int knight_pcsq_mg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,-40,-20,-10,-10,-10,-10,-20, -40, 0,  // A1 ... H1  
   0,-20, -5, -5, -5, -5, -5, -5, -20, 0,  // A2 ... H2 
   0,-10, -5,  5,  5,  5,  5, -5, -10, 0,  // A3 ... H3 
   0,-10, -5, 10, 15, 15, 10, -5, -10, 0,  // A4 ... H4 
   0,-15, -5, 10, 15, 15, 10, -5, -15, 0,  // A5 ... H5 
   0,-15,  0,  5, 10, 10,  5,  0, -15, 0,  // A6 ... H6 
   0,-40,-20,  0,  5,  5,  0,-20, -40, 0,  // A7 ... H7 
   0,-140,-40,-20,-20,-20,-20,-40,-140, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int knight_pcsq_eg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,-40,-20,-10,-10,-10,-10,-20, -40, 0,  // A1 ... H1  
   0,-20, -5, -5, -5, -5, -5, -5, -20, 0,  // A2 ... H2 
   0,-10, -5,  5,  5,  5,  5, -5, -10, 0,  // A3 ... H3 
   0,-10, -5, 10, 15, 15, 10, -5, -10, 0,  // A4 ... H4 
   0,-15, -5, 10, 15, 15, 10, -5, -15, 0,  // A5 ... H5 
   0,-15,  0,  5, 10, 10,  5,  0, -15, 0,  // A6 ... H6 
   0,-40,-20,  0,  5,  5,  0,-20, -40, 0,  // A7 ... H7 
   0,-140,-40,-20,-20,-20,-20,-40,-140, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int bishop_pcsq_mg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,-10,-10,-10,-10,-10,-10,-10,-10, 0,  // A1 ... H1  
   0, -5, 10, -5, -5, -5, -5, 10, -5, 0,  // A2 ... H2 
   0,-10, -5,  5,  0,  0,  5, -5,-10, 0,  // A3 ... H3 
   0,-10, -2,  0, 10, 10,  0, -2,-15, 0,  // A4 ... H4 
   0,-10, -2,  0, 10, 10,  0, -2,-10, 0,  // A5 ... H5 
   0,-10, -5,  5,  0,  0,  5, -5,-10, 0,  // A6 ... H6 
   0,-10,-10, -5, -5, -5, -5,-10,-10, 0,  // A7 ... H7 
   0,-15,-10,-10,-10,-10,-10,-10,-15, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int bishop_pcsq_eg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,-10,-10,-10,-10,-10,-10,-10,-10, 0,  // A1 ... H1  
   0, -5, 10, -5, -5, -5, -5, 10, -5, 0,  // A2 ... H2 
   0,-10, -5,  5,  0,  0,  5, -5,-10, 0,  // A3 ... H3 
   0,-10, -2,  0, 10, 10,  0, -2,-15, 0,  // A4 ... H4 
   0,-10, -2,  0, 10, 10,  0, -2,-10, 0,  // A5 ... H5 
   0,-10, -5,  5,  0,  0,  5, -5,-10, 0,  // A6 ... H6 
   0,-10,-10, -5, -5, -5, -5,-10,-10, 0,  // A7 ... H7 
   0,-15,-10,-10,-10,-10,-10,-10,-15, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

   const int rook_pcsq_mg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0,
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0,
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A1 ... H1  
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A2 ... H2 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A3 ... H3 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A4 ... H4 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A5 ... H5 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A6 ... H6 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A7 ... H7 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A8 ... H8
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0,
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0} ;

  const int rook_pcsq_eg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0,
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0,
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A1 ... H1  
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A2 ... H2 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A3 ... H3 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A4 ... H4 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A5 ... H5 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A6 ... H6 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A7 ... H7 
   0, -5,-2,  0,  4,  4,  0, -2, -5, 0,  // A8 ... H8
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0,
   0,  0, 0,  0,  0,  0,  0,  0,  0, 0} ;

 const int queen_pcsq_mg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0, -5, -5, -5, -5, -5, -5, -5, -5, 0,  // A1 ... H1  
   0, -5,  0,  0,  0,  0,  0,  0, -5, 0,  // A2 ... H2 
   0, -5,  0,  0,  0,  0,  0,  0, -5, 0,  // A3 ... H3 
   0, -5,  0,  0,  0,  0,  0,  0, -5, 0,  // A4 ... H4 
   0, -5,  0,  0,  0,  0,  0,  0, -5, 0,  // A5 ... H5 
   0, -5,  0,  0,  0,  0,  0,  0, -5, 0,  // A6 ... H6 
   0, -5,  0,  0,  0,  0,  0,  0, -5, 0,  // A7 ... H7 
   0, -5, -5, -5, -5, -5, -5, -5, -5, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int queen_pcsq_eg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,-30,-20,-10, -8, -8,-10,-20,-30, 0,  // A1 ... H1  
   0,-20, -8, -4,  0,  0, -4, -8,-20, 0,  // A2 ... H2 
   0,-10, -4,  0,  4,  4,  0, -4,-10, 0,  // A3 ... H3 
   0, -8,  0,  0,  0,  0,  0,  0, -8, 0,  // A4 ... H4 
   0, -8,  0,  0,  0,  0,  0,  0, -8, 0,  // A5 ... H5 
   0 -10,  0,  0,  0,  0,  0,  0,-10, 0,  // A6 ... H6 
   0,-20, -8, -4,  0,  0, -4, -8,-20, 0,  // A7 ... H7 
   0,-30,-20,-10, -8, -8,-10,-20,-30, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int king_pcsq_mg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0, 30, 40,  0,  0, 10,  0, 50, 40, 0,  // A1 ... H1  
   0, 10, 20,  0,  0,  0,  0, 20, 10, 0,  // A2 ... H2 
   0,10, 10,  0,-20,-20,  0, 10, 10, 0,  // A3 ... H3 
   0,-10, -5,-20,-40,-40,-20, -5,-10, 0,  // A4 ... H4 
   0,-30,-20,-40,-60,-60,-40,-20,-30, 0,  // A5 ... H5 
   0,-80,-80,-80,-80,-80,-80,-80,-80, 0,  // A6 ... H6 
   0,-80,-80,-80,-80,-80,-80,-80,-80, 0,  // A7 ... H7 
   0,-80,-80,-80,-80,-80,-80,-80,-80, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

const int king_pcsq_eg[120]= {
 //    A   B   C   D   E   F   G   H  
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,-30,-20,-10, -5, -5,-10,-20,-30, 0,  // A1 ... H1  
   0,-20,-10,  0, 10, 10,  0,-10,-20, 0,  // A2 ... H2 
   0,-10,  0, 10, 20, 20, 10,  0,-10, 0,  // A3 ... H3 
   0,  0, 10, 20, 30, 30, 20, 10,  0, 0,  // A4 ... H4 
   0,  0, 10, 20, 30, 30, 20, 10,  0, 0,  // A5 ... H5 
   0,-10,  0, 10, 20, 20, 10,  0,-10, 0,  // A6 ... H6 
   0,-20,-10,  0, 10, 10,  0,-10,-20, 0,  // A7 ... H7 
   0,-30,-20,-10, -5, -5,-10,-20,-30, 0,  // A8 ... H8
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 0} ;

// Castigo para el peon, dependiendo de la columna

//***********************************************************
static const int aislado_mg[10]   = {0,12, 14, 14, 18, 18, 14, 14, 12,0};
static const int aislado_eg[10]   = {0,10, 12, 12, 14, 14, 12, 12, 10,0};

static const int retrasado_mg[10] = {0,10, 10, 10, 12, 12, 10, 10, 10,0};
static const int retrasado_eg[10] = {0, 8,  8,  8, 10, 10,  8,  8,  8,0};

static const int doblado_mg[10]   = {0, 8,  8,  8, 10, 10,  8,  8,  8,0};
static const int doblado_eg[10]   = {0,14, 15, 15, 16, 16, 15, 15, 14,0};

static const int candidato_mg[8] =  {0, 0, 4,  6, 12, 30, 0, 0};
static const int candidato_eg[8] =  {0, 0, 8, 12, 24, 60, 0, 0};

static const int conectado_mg[10] =     {0,4, 4, 4, 5, 5, 4, 4, 4,0};
static const int conectado_eg[10] =     {0,2, 2, 2, 4, 4, 2, 2, 2,0};

static const int castigo = 8;
static const int pp_conectado = 4;
static const int bonus_pp[8] =      { 0,  0,  0, 8, 24, 48, 86, 116};//ori 3
static const int bonus_rey[8] =     {50, 30, 15, 5,  0,  0,  0,   0};

//***********************************************************************

 int Valor_FichasB;
 int Valor_FichasN;
 int Fila_Ultimo_PeonB[10];
 int Fila_Ultimo_PeonN[10];
 int Lista_FichasB[17], Lista_FichasN[17];
 int Num_FiB, Num_FiN;
 int     dist_para_coronarB;
 int     dist_para_coronarN;


void  init_pcsq(void){
int x;
for( x = 0; x <= 119; x++){
wp_pcsq_mg[x] = pawn_pcsq_mg[x];
bp_pcsq_mg[x] = pawn_pcsq_mg[FLIP[x]];
wn_pcsq_mg[x] = knight_pcsq_mg[x];
bn_pcsq_mg[x] = knight_pcsq_mg[FLIP[x]];
wb_pcsq_mg[x] = bishop_pcsq_mg[x];
bb_pcsq_mg[x] = bishop_pcsq_mg[FLIP[x]];
wr_pcsq_mg[x] = rook_pcsq_mg[x];
br_pcsq_mg[x] = rook_pcsq_mg[FLIP[x]];
wq_pcsq_mg[x] = queen_pcsq_mg[x];
bq_pcsq_mg[x] = queen_pcsq_mg[FLIP[x]];
wk_pcsq_mg[x] = king_pcsq_mg[x];
bk_pcsq_mg[x] = king_pcsq_mg[FLIP[x]];

//wp_pcsq_eg[x] = pawn_pcsq_eg[x];
//bp_pcsq_eg[x] = pawn_pcsq_eg[FLIP[x]];
wn_pcsq_eg[x] = knight_pcsq_eg[x];
bn_pcsq_eg[x] = knight_pcsq_eg[FLIP[x]];
wb_pcsq_eg[x] = bishop_pcsq_eg[x];
bb_pcsq_eg[x] = bishop_pcsq_eg[FLIP[x]];
wr_pcsq_eg[x] = rook_pcsq_eg[x];
br_pcsq_eg[x] = rook_pcsq_eg[FLIP[x]];
wq_pcsq_eg[x] = queen_pcsq_eg[x];
bq_pcsq_eg[x] = queen_pcsq_eg[FLIP[x]];
wk_pcsq_eg[x] = king_pcsq_eg[x];
bk_pcsq_eg[x] = king_pcsq_eg[FLIP[x]];
} //for
} // init

void init_peones(void){
int ca,col,x;
Num_FiB = Num_FiN = 0;

for(ca = 0; ca < 17; ca++){
Lista_FichasB[ca]= 0;
Lista_FichasN[ca]= 0;}

for (col=0; col<10; col++){
   Fila_Ultimo_PeonB[col] = 8;
   Fila_Ultimo_PeonN[col] = 1;}

	for(x = 0; x < 64; x++){ 
	if(contenido[BOARD_64[x]] == VACIO) //BOARD_64
	continue;
	ca = BOARD_64[x]; //BOARD_64
	if(contenido[ca] > 0 ){
	   Lista_FichasB[Num_FiB++] = ca;
	   if(contenido[ca] == P_B){
		   col = COLUMNA[ca];
		   if(FILA[ca] < Fila_Ultimo_PeonB[col])
              Fila_Ultimo_PeonB[col] = FILA[ca];
	   }
	   else if(contenido[ca] == D_B) DB = ca;
	}// ca > 0

	   else if (contenido[ca] < 0){
           Lista_FichasN[Num_FiN++] = ca;
	       if(contenido[ca] == P_N){
		   col = COLUMNA[ca];
		   if(FILA[ca] > Fila_Ultimo_PeonN[col])
              Fila_Ultimo_PeonN[col]= FILA[ca];
		   }
		   else if(contenido[ca] == D_N) DN = ca;
	  }// ca < 0
	 
} // for ca
 return;
}


int EvaluaPeonB(int ca) {
int score = 0;
int col  = COLUMNA[ca];
int fila = FILA[ca];

bool expuesto,doblado,aislado,conectado,pasado,retrasado;
bool primero = false;

 score  += wp_pcsq_mg[ca];
 expuesto = (Fila_Ultimo_PeonN[col] < fila); 

 if(expuesto){ // pasarlo a pasado y candidato, y if(exp && fila < 7) penalizacion
	 primero = true;
	 switch (fila) {	
	 case 2: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B)||(contenido[ca+30] == P_B)||
				(contenido[ca+40] == P_B)||(contenido[ca+50] == P_B))primero = false;
		 break;
	 case 3: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B)||
				 (contenido[ca+30] == P_B)||(contenido[ca+40] == P_B))primero = false;
		 break;
	 case 4: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B)||
				 (contenido[ca+30] == P_B))primero = false;
		 break;
	 case 5: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B))primero = false;
		 break;
	 case 6: if((contenido[ca+10] == P_B))primero = false;
		 break;
	 default : break;
	 }
	 if(!primero)expuesto = false;
 }//expuesto
 
 doblado = Fila_Ultimo_PeonB[col] < fila;
 aislado = (Fila_Ultimo_PeonB[col-1] == 8 &&
            Fila_Ultimo_PeonB[col+1] == 8);
 conectado = (pb_controla(ca+10) || pb_controla(ca));
 pasado    = (expuesto) &&
	         (Fila_Ultimo_PeonN[col-1] <= fila && Fila_Ultimo_PeonN[col+1] <= fila);
 retrasado = !aislado && !conectado && !pasado &&
             (Fila_Ultimo_PeonB[col-1] > fila && Fila_Ultimo_PeonB[col+1] > fila);
    
    
 // si no esta expuesto y es la base de una cadena o
 // podemos avanzar y conectar con otro peon, no sera retrasado   
 if(retrasado && !expuesto){
	if((contenido[ca+11] == P_B || contenido[ca+9] == P_B))
		retrasado = false;
	else if(contenido[ca+10] == VACIO &&
	  (contenido[ca+21] == P_B || contenido[ca+19] == P_B) &&
	  !pn_controla(ca+10) ) retrasado = false;
 }//retrasado

 if(retrasado){
	score -= retrasado_mg[col];
	if(expuesto)score -= castigo;
	//if(pn_controla(ca+10)) score -= 8;
 }
 if (doblado) score -= doblado_mg[col];
 if(aislado){
     score  -= aislado_mg[col];
     if(expuesto)score -= castigo; 
 }
 if(conectado) score  += conectado_mg[col];

 if(pasado){
	score += bonus_pp[fila];
    if(conectado) score += pp_conectado * fila;	
	 //if(contenido[ca+10] != VACIO)score -= 8;
	//else if(!aislado)score += 8;
 }
 else if(expuesto  && conectado && fila > 2 && fila < 6){
	 int i;
	 bool candidato = true;
	    for(i = ca+10; i < (A6); i+=10){
		if(pb_controla(i) < pn_controla(i)){
			candidato = false;
			break;
		}
	   }//for
       if(candidato) score += candidato_mg[fila];
}//candidato
 return score;
}
    
int EvaluaPeonBF(int ca) {
int score = 0,i;
int col  = COLUMNA[ca];
int fila = FILA[ca];
bool expuesto,doblado,aislado,conectado,pasado,retrasado;
bool primero = false;
 
expuesto = (Fila_Ultimo_PeonN[col] < fila); 
 if(expuesto){ // pasarlo a pasado y candidato, y if(exp && fila < 7) penalizacion
	 primero = true;
	 switch (fila) {	
	 case 2: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B)||(contenido[ca+30] == P_B)||
				(contenido[ca+40] == P_B)||(contenido[ca+50] == P_B))primero = false;
		 break;
	 case 3: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B)||
				 (contenido[ca+30] == P_B)||(contenido[ca+40] == P_B))primero = false;
		 break;
	 case 4: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B)||
				 (contenido[ca+30] == P_B))primero = false;
		 break;
	 case 5: if((contenido[ca+10] == P_B)||(contenido[ca+20] == P_B))primero = false;
		 break;
	 case 6: if((contenido[ca+10] == P_B))primero = false;
		 break;
	 default : break;
	 }
	 if(!primero) expuesto = false;
 }//expuesto
 
 doblado = Fila_Ultimo_PeonB[col] < fila;
 aislado = (Fila_Ultimo_PeonB[col-1] == 8 &&
            Fila_Ultimo_PeonB[col+1] == 8);
 conectado = (pb_controla(ca+10) || pb_controla(ca));
 pasado    = (expuesto && primero) &&
	         (Fila_Ultimo_PeonN[col-1] <= fila && Fila_Ultimo_PeonN[col+1] <= fila);
 retrasado = !aislado && !conectado && !pasado &&
             (Fila_Ultimo_PeonB[col-1] > fila && Fila_Ultimo_PeonB[col+1] > fila);
    
 
 // si no esta expuesto y es la base de una cadena o
 // podemos avanzar y conectar con otro peon, no sera retrasado   
 if(retrasado && !expuesto){
	if((contenido[ca+11] == P_B || contenido[ca+9] == P_B))
		retrasado = false;
	else if(contenido[ca+10] == VACIO &&
	  (contenido[ca+21] == P_B || contenido[ca+19] == P_B) &&
	  !pn_controla(ca+10) ) retrasado = false;
 }//retrasado

 if(retrasado){
	score -= retrasado_eg[col];
	if(expuesto)score -= castigo; 
 }
 if (doblado) score -= doblado_eg[col];
 if(aislado){
     score  -= aislado_eg[col];
     if(expuesto)score -= castigo; 
 }
 if(conectado) score  += conectado_eg[col];
 
 if(pasado){
	 int dist_peon = 8-fila;
	 int dist_rn =   Distancia[Pos_RN][CORONA_B[ca]];

	 score += bonus_pp[fila]*2;
	 if(conectado) score += pp_conectado * fila *2;	
	 if(contenido[ca+10] == VACIO) {
		 if(pb_controla(ca+10) || Distancia[Pos_RB][ca+10] == 1)
			 score += 6 * fila;
		 else score += 3 * fila;
	 }
	 //else if(!aislado)score += 16;

	 if(FichasNN == 0) {
		 //Puede coronar solo
		if(Color == NEGRAS)dist_peon++;
		if(fila == 2 && (contenido[ca+10] == VACIO &&
			contenido[ca+20] == VACIO)) dist_peon--;
		//Tiene delante algo que le bloquea el paso ?
		for(i=ca+10; i < (A8+col); i += 10){
		if(contenido[i] != VACIO) dist_peon++;
		}
		if(dist_peon < dist_rn){
		dist_para_coronarB = Min(dist_peon,dist_para_coronarB);
		}	
	 }// FN=0
	
	 score  += (bonus_rey[Distancia[Pos_RB][ca+10]] - 
                bonus_rey[Distancia[Pos_RN][ca+10]]);
 }//pasado

 else if(expuesto  && conectado && fila > 2 && fila < 6){
	 bool candidato = true;
	    for(i = ca+10; i < (A6); i+=10){
		if(pb_controla(i) < pn_controla(i)){
			candidato = false;
			break;
		}
	   }//for
       if(candidato) score += candidato_eg[fila];
	   
}//candidato
 return score;
}

int EvaluaPeonN(int ca) {
int score = 0;
int col  = COLUMNA[ca];
int fila = FILA[ca];
int fila_inv = (9-fila);
bool expuesto,doblado,aislado,conectado,pasado,retrasado;
bool primero = false;

 score += bp_pcsq_mg[ca];
 expuesto = (Fila_Ultimo_PeonB[col] > fila); 
 if(expuesto){ 
	 primero = true;
	 switch (fila_inv) {	
	 case 2: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N)||(contenido[ca-30] == P_N)||
				(contenido[ca-40] == P_N)||(contenido[ca-50] == P_N))primero = false;
		 break;
	 case 3: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N)||
				 (contenido[ca-30] == P_N)||(contenido[ca-40] == P_N))primero = false;
		 break;
	 case 4: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N)||
				 (contenido[ca-30] == P_N))primero = false;
		 break;
	 case 5: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N))primero = false;
		 break;
	 case 6: if((contenido[ca-10] == P_N))primero = false;
		 break;
	 default : break;
	 }
	 if(!primero) expuesto = false;
 }//expuesto
 doblado  = (Fila_Ultimo_PeonN[col] > fila);
 aislado = (Fila_Ultimo_PeonN[col-1] == 1 &&
            Fila_Ultimo_PeonN[col+1] == 1);
 conectado = (pn_controla(ca-10) || pn_controla(ca));
 pasado    = (expuesto)&&
	         (Fila_Ultimo_PeonB[col-1] >= fila && Fila_Ultimo_PeonB[col+1] >= fila);
 retrasado = !aislado && !conectado && !pasado &&
	         (Fila_Ultimo_PeonN[col-1] < fila && Fila_Ultimo_PeonN[col+1] < fila );
           
 // si es la base de una cadena o
 // podemos avanzar y conectar con otro peon, no sera retrasado   
 if(retrasado && !expuesto){
	if((contenido[ca-11] == P_N || contenido[ca-9] == P_N))
		retrasado = false;
	else if(contenido[ca-10] == VACIO &&
	  (contenido[ca-21] == P_N || contenido[ca-19] == P_N) &&
	  !pb_controla(ca-10) ) retrasado = false;
 }//retrasado
 
 if(retrasado){
	score -= retrasado_mg[col];
	if(expuesto)score -= castigo;
 }
 if (doblado) score -= doblado_mg[col];
 if(aislado){
     score  -= aislado_mg[col];
     if(expuesto)score -= castigo; 
 }
 if(conectado) score  += conectado_mg[col];

 if(pasado){
	score += bonus_pp[fila_inv];
    if(conectado) score += pp_conectado * fila_inv;	
	//else if(!aislado)score += 8;
	//if(contenido[ca-10] != VACIO)score -= 8;
 }
 else if(expuesto  && conectado && fila_inv > 2 && fila_inv < 6){
	 int i;
	 bool candidato = true;
	    for(i = ca-10; i > (H3); i-=10){
		if(pn_controla(i) < pb_controla(i)){
			candidato = false;
			break;
		}
	   }//for
       if(candidato) score += candidato_mg[fila_inv];
}//candidato
 return score;
}

int EvaluaPeonNF(int ca) {
int score = 0,i;
int col  = COLUMNA[ca];
int fila = FILA[ca];
int fila_inv = (9-fila);
bool expuesto,doblado,aislado,conectado,pasado,retrasado;
bool primero;

 expuesto = (Fila_Ultimo_PeonB[col] > fila); 
if(expuesto){ 
	 primero = true;
	 switch (fila_inv) {	
	 case 2: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N)||(contenido[ca-30] == P_N)||
				(contenido[ca-40] == P_N)||(contenido[ca-50] == P_N))primero = false;
		 break;
	 case 3: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N)||
				 (contenido[ca-30] == P_N)||(contenido[ca-40] == P_N))primero = false;
		 break;
	 case 4: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N)||
				 (contenido[ca-30] == P_N))primero = false;
		 break;
	 case 5: if((contenido[ca-10] == P_N)||(contenido[ca-20] == P_N))primero = false;
		 break;
	 case 6: if((contenido[ca-10] == P_N))primero = false;
		 break;
	 default : break;
	 }
	 if(!primero) expuesto = false;
 }//expuesto

 doblado   = (Fila_Ultimo_PeonN[col] > fila);
 aislado   = (Fila_Ultimo_PeonN[col-1] == 1 &&
              Fila_Ultimo_PeonN[col+1] == 1);
 conectado = (pn_controla(ca-10) || pn_controla(ca));
 pasado    = (expuesto)&&
	         (Fila_Ultimo_PeonB[col-1] >= fila && Fila_Ultimo_PeonB[col+1] >= fila);
 retrasado = !aislado && !conectado && !pasado &&
	         (Fila_Ultimo_PeonN[col-1] < fila && Fila_Ultimo_PeonN[col+1] < fila );
             
   
 if(retrasado && !expuesto){
	if((contenido[ca-11] == P_N || contenido[ca-9] == P_N))
		retrasado = false;
	else if(contenido[ca-10] == VACIO &&
	  (contenido[ca-21] == P_N || contenido[ca-19] == P_N) &&
	  !pb_controla(ca-10) ) retrasado = false;
 }//retrasado
 
 if(retrasado){
	score -= retrasado_eg[col];
	if(expuesto)score -= castigo;
 }
 if (doblado) score -= doblado_eg[col];
 if(aislado){
     score  -= aislado_eg[col];
     if(expuesto)score -= castigo; 
 }
 if(conectado) score  += conectado_eg[col];
 
 if(pasado){
	 int dist_peon = 8-fila_inv;
	 int dist_rb =   Distancia[Pos_RB][CORONA_N[ca]];
	 
	score += bonus_pp[fila_inv]*2;
    if(conectado) score += pp_conectado * fila_inv * 2;	
	//else if(!aislado)score += 16;
	if(contenido[ca-10] == VACIO) {
		 if(pn_controla(ca-10) || Distancia[Pos_RN][ca-10] == 1)
		 score += 6 * fila_inv;
		 else score += 3 * fila_inv;
	 }

    if(FichasBB == 0){
		int dist_peon = Distancia[ca][CORONA_N[ca]];
		int dist_rb =   Distancia[Pos_RB][CORONA_N[ca]];

		// Puede coronar solo 
		if(Color == BLANCAS)dist_peon++;
		if(fila_inv == 2 && (contenido[ca-10] == VACIO &&
		contenido[ca-20] == VACIO)) dist_peon--;

		//Tiene delante algo que le bloquea el paso ?
		for(i=ca-10; i >= CORONA_N[ca]; i -= 10){
		if(contenido[i] != VACIO) dist_peon++;
		}

		if(dist_peon < dist_rb){
		dist_para_coronarN = Min(dist_peon,dist_para_coronarN);
		//PPCSN++;
		}
	}//FB = 0
	
	score  += (bonus_rey[Distancia[Pos_RN][ca-10]] - 
               bonus_rey[Distancia[Pos_RB][ca-10]]);
 }// pasado

 else if(expuesto  && conectado && fila_inv > 2 && fila_inv < 6){
	 int i;
	 bool candidato = true;
	    for(i = ca-10; i > (H3); i-=10){
		if(pn_controla(i) < pb_controla(i)){
			candidato = false;
			break;
		}
	   }//for
       if(candidato) score += candidato_eg[fila_inv];
}//candidato
 return score;
}





