

//** Para las tablas hash, he intentado adaptar las de Danasah, de Pedro Castro,
//** Ultima revision, verano del 2015.
//** http://sites.google.com/site/danasah

/*
A 32 bit random number generator. An implementation in C of the algorithm given by
Knuth, the art of computer programming, vol. 2, pp. 26-27. We use e=32, so
we have to evaluate y(n) = y(n - 24) + y(n - 55) mod 2^32, which is implicitly
done by unsigned arithmetic. */

//random numbers from Mathematica 2.0.
unsigned int Rand32()
{

  static const unsigned long x[55] =
  {
    1410651636UL, 3012776752UL, 3497475623UL, 2892145026UL, 1571949714UL,
    3253082284UL, 3489895018UL, 387949491UL,  2597396737UL, 1981903553UL,
    3160251843UL, 129444464UL,  1851443344UL, 4156445905UL, 224604922UL,
    1455067070UL, 3953493484UL, 1460937157UL, 2528362617UL, 317430674UL,
    3229354360UL, 117491133UL,  832845075UL,  1961600170UL, 1321557429UL,
    747750121UL,  545747446UL,  810476036UL,  503334515UL,  4088144633UL,
    2824216555UL, 3738252341UL, 3493754131UL, 3672533954UL, 29494241UL,
    1180928407UL, 4213624418UL, 33062851UL,   3221315737UL, 1145213552UL,
    2957984897UL, 4078668503UL, 2262661702UL, 65478801UL,   2527208841UL,
    1960622036UL, 315685891UL,  1196037864UL, 804614524UL,  1421733266UL,
    2017105031UL, 3882325900UL, 810735053UL,  384606609UL,  2393861397UL
  };

  static int init = 1;
  static unsigned long y[55];
  static int j, k;
  unsigned long ul;

  if (init)
  {
    int i;

    init = 0;

    for (i = 0; i < 55; i++) y[i] = x[i];

    j = 24 - 1;
    k = 55 - 1;
  }

  ul = (y[k] += y[j]);

  if (--j < 0) j = 55 - 1;

  if (--k < 0) k = 55 - 1;

  return ((unsigned int)ul);
}

U64 Rand64(void)
{
  U64 result;
  unsigned int r1, r2;

  r1 = Rand32();
  r2 = Rand32();
  result = r1 | (U64) r2 << 32;
  return (result);
}

void init_hash()
{
  int ficha, casilla;
  srand(0);

  for (ficha = 0; ficha < 14; ficha++)
    for (casilla = 0; casilla < H9; casilla++)
      zobrist[ficha][casilla] = Rand64();

  hash_turno = Rand64();

  for (casilla = 0; casilla < H9; casilla++)
    hash_alpaso[casilla] = Rand64();

  hash = 0;

}

//* a todas las piezas le sumamos 7 para convertir las
//* negras a numeros positivos, as el Rey negro (-6)
//* pasar a ser (-6) + (7) = 1

void set_hash()
{
  register int casilla;

  hash = 0;

  for (casilla = A1; casilla < H9; casilla++)
  {
    if (contenido[casilla] != VACIO &&
        contenido[casilla] != FUERA)
      hash = hash ^ zobrist[contenido[casilla] + 7][casilla];
  }

  if (Color == BLANCAS) hash ^= hash_turno;

  if (Alpaso != 0)
    hash ^= hash_alpaso[Alpaso];
}

bool repeticion()
{
  int i;

  for (i = hply - fifty; i < hply; ++i)
  {
    if (historial[i].hash == hash)
      return true;
  }

  return false;
}


TType *TTable;
EType *ETable;

// limpiamos las tablas hash (search y eval)
void clear_hash(void)
{
  memset(TTable, 0, sizeof(TType) * TTSize);
  memset(ETable, 0, sizeof(EType) * ETSize);
  Trans_Age = 0;
};


// almacenamos informacin en las tablas hash
void store_hash(int depth, int value, int flag, tipomov mov)
{

  int index = hash % TTSize;

  if (TTable[index].age == Trans_Age &&
      TTable[index].Depth > depth)
    return;

  if (abs (value) > VALMATE - 100)
  {
    if (value > 0) value += Profundidad;

    else value -= Profundidad;
  }

  TTable[index].Hash  = hash;
  TTable[index].Type  = flag;
  TTable[index].Depth = depth;
  TTable[index].Valor = value;
  TTable[index].age   = Trans_Age;
  TTable[index].Mov   = mov;

  return;
}

void Get_hash_move(tipomov *hashmov)
{
  int index;
  index = hash % TTSize;
  *hashmov = nomove;

  if (TTable[index].Hash == hash)
  {
    *hashmov = TTable[index].Mov;
  }
}


// probamos si la posicin la tenemos almacenada en las tablas hash
int  probe_hash(int depth, int *value, int beta, tipomov *hashmov, U8 *tt_null_ok)
{

  int index;

  Hash_Probe++;
  index = hash % TTSize;
  *hashmov = nomove;

  if (TTable[index].Hash == hash)
  {
    *hashmov = TTable[index].Mov;
    *value = TTable[index].Valor;

    if (TTable[index].Depth >= (depth - (4 * UNPLY)) &&
        *value < beta && TTable[index].Type == Hash_ALPHA)
      *tt_null_ok = false;

    if (TTable[index].Depth >= depth)
    {
      Hash_Hits++;

      if (abs (*value) > VALMATE - 100)
      {
        if (*value > 0) *value -= Profundidad;

        else            *value += Profundidad;
      }

      return TTable[index].Type;
    }//if Depth
  }//if Hash

  // No se encontr
  return Hash_DESCONOCIDO;
}


// almacenamos informacin sobre la evaluacin de una posicin
void store_eval(int score, int turno)
{

  int index = hash % ETSize;

  ETable[index].Hash = hash;
  ETable[index].Valor = score;
  ETable[index].Color = turno;
  return;
}

// comprobamos si una posicin ya la hemos evaluado antes
int probe_eval(int turno)
{

  int index = hash % ETSize;

  if (ETable[index].Hash == hash &&
      ETable[index].Color == turno)
  {
    return (ETable[index].Valor);
  }

  return -VALMATE;
}

// asignamos memoria para las tablas hash y evaluacin cach
void alloc_hash(void)
{
  TTable = (TType *) malloc(sizeof(TType) * TTSize);
  ETable = (EType *) malloc(sizeof(EType) * ETSize);

  if (TTable == NULL || ETable == NULL )
  {
    printf("ERROR: Couldn't allocate memory for hash table.\n");
    exit(1);
  }

  return;
}


// liberamos el espacio ocupado por las tablas hash
// cuando salimos del programa
void free_hash(void)
{
  free(TTable);
  free(ETable);
  return;
}

